﻿using Microsoft.Win32.SafeHandles;
using System;
using System.Runtime.InteropServices;
using System.IO.Ports;
using System.Reflection;
using System.Windows.Forms;

namespace WindowsFormsApplication1
{


    public partial class Commo : Form
    {

        [StructLayout(LayoutKind.Sequential)]
        internal struct DCB
        {
            internal uint DCBLength;
            internal uint BaudRate;
            private UInt32 Flags;

            private ushort wReserved;        // not currently used
            internal ushort XonLim;           // transmit XON threshold
            internal ushort XoffLim;          // transmit XOFF threshold             

            internal byte ByteSize;
            internal Parity Parity;
            internal StopBits StopBits;

            internal sbyte XonChar;          // Tx and Rx XON character
            internal sbyte XoffChar;         // Tx and Rx XOFF character
            internal sbyte ErrorChar;        // error replacement character
            internal sbyte EofChar;          // end of input character
            internal sbyte EvtChar;          // received event character
            private ushort wReserved1;       // reserved; do not use     
        }

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool SetCommState(IntPtr hFile, [In] ref DCB lpDCB);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool GetCommState(IntPtr hFile, [In, Out] ref DCB lpDCB);

        public delegate void AddTxRxText(char[] d, int n);
        public AddTxRxText trxDelegate;

        public Commo()
        {
            InitializeComponent();
            trxDelegate = new AddTxRxText(AddTxRxTextMethod);
            string[] ports = System.IO.Ports.SerialPort.GetPortNames();
            if (ports.Length > 0)
                foreach (string S in ports)
                    comboBoxCOMPort.Items.Add(S);
            else
                comboBoxCOMPort.Items.Add("No Serial Ports");
            comboBoxCOMPort.SelectedIndex = 0;
            comboBoxBaudRate.SelectedIndex = 2;
            comboBoxSettings.SelectedIndex = 0;
        }
    
        public void AddTxRxTextMethod(char[] d, int n)
        {
            string S = new string(d, 0, n);
            textBoxDisplayRich.SelectionColor = System.Drawing.Color.FromName("Green");
            textBoxDisplayRich.AppendText(S);
            textBoxDisplayRich.SelectionColor = System.Drawing.Color.FromName("Black");
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            System.IO.Ports.SerialPort P = sender as System.IO.Ports.SerialPort;
            int n = P.BytesToRead;
            char[] d = new char[n];
            P.Read(d, 0, n);
            BeginInvoke(trxDelegate, new Object[]{d, n}); 
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen)
            {
                switch (comboBoxSettings.SelectedIndex) 
                {
                    case 0: // 7E1
                        serialPort1 = new SerialPort(comboBoxCOMPort.Text, Convert.ToInt32(comboBoxBaudRate.Text), Parity.Even, 7, StopBits.One);
                        break;
                    case 1: // 8N1
                        serialPort1 = new SerialPort(comboBoxCOMPort.Text, Convert.ToInt32(comboBoxBaudRate.Text), Parity.None, 8, StopBits.One);
                        break;
                    case 2: // 9-bit mode
                        serialPort1 = new SerialPort(comboBoxCOMPort.Text, Convert.ToInt32(comboBoxBaudRate.Text), Parity.None, 8, StopBits.One);
                        break;

                }
                serialPort1.Handshake = System.IO.Ports.Handshake.None;
                serialPort1.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(serialPort1_DataReceived);
                serialPort1.ReadTimeout = 500;
                serialPort1.WriteTimeout = 500;
                serialPort1.NewLine = "\r\n";
                serialPort1.Open();
                if (comboBoxSettings.SelectedIndex == 2 )
                {
                    System.IO.Stream stream = serialPort1.BaseStream;
                    var handle = (SafeFileHandle)stream.GetType().GetField("_handle", BindingFlags.NonPublic | BindingFlags.Instance).GetValue(stream);
                    DCB Dcb = new DCB();
                    bool status = GetCommState(handle.DangerousGetHandle(), ref Dcb);
                    if (false == status)
                    {
                        int error = Marshal.GetLastWin32Error();
                        Console.WriteLine("GetCommState Error was: " + error);
                    }
                    Dcb.ByteSize = 9;
                    status = SetCommState(handle.DangerousGetHandle(), ref Dcb);
                    if (false == status)
                    {
                        int error = Marshal.GetLastWin32Error();
                        Console.WriteLine("SetCommState Error was: " + error);
                    }
                    status = GetCommState(handle.DangerousGetHandle(), ref Dcb);
                    if (false == status)
                    {
                        int error = Marshal.GetLastWin32Error();
                        Console.WriteLine("GetCommState Error was: " + error);
                    }
                    Console.WriteLine(Dcb.ToString());
                }



                buttonConnect.Text = "Disconnect";

                textBoxTXData.Enabled = true;
            }else{
                buttonConnect.Text = "Connect";
                serialPort1.Close();
                textBoxTXData.Enabled = false;
            }
        }

        private void textBoxTXData_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return) 
            {
                serialPort1.WriteLine(textBoxTXData.Text);
                textBoxDisplayRich.SelectionColor = System.Drawing.Color.FromName("Blue");
                textBoxDisplayRich.AppendText(textBoxTXData.Text + '\r' + '\n');
                textBoxDisplayRich.SelectionColor = System.Drawing.Color.FromName("Black");
                textBoxTXData.SelectAll(); 
                e.Handled = true;
            }
        }
    }
}
